﻿using Moq;
using NUnit.Framework;
using Sunset.Interface;
using Sunset.Library;
using System;

namespace SunsetLibrary.Tests
{
    [TestFixture]
    public class SunsetCalculatorTests
    {
        private Mock<ISolarService> GetMockService()
        {
            string goodResult = "{\"results\":{\"sunrise\":\"6:39:53 AM\",\"sunset\":\"6:48:32 PM\",\"solar_noon\":\"12:44:12 PM\",\"day_length\":\"12:08:38.8300000\"},\"status\":\"OK\"}";

            Mock<ISolarService> serviceMock = new Mock<ISolarService>();
            serviceMock.Setup(s => s.GetServiceData(It.IsAny<DateTime>()))
                .Returns(goodResult);
            return serviceMock;
        }

        [Test]
        public void SunsetCalculator_OnCreation_MatchesInterface()
        {
            var calculator = new SunsetCalculator();
            Assert.IsInstanceOf<ISunsetCalculator>(calculator);
        }

        [Test]
        public void GetSunset_OnValidDate_ReturnsExpectedTime()
        {
            Mock<ISolarService> serviceMock = GetMockService();

            var calculator = new SunsetCalculator();
            calculator.Service = serviceMock.Object;

            DateTime targetDate = new DateTime(2016, 09, 22);
            DateTime expected = new DateTime(2016, 09, 22, 18, 48, 32);

            DateTime result = calculator.GetSunset(targetDate);

            Assert.AreEqual(expected, result);
        }

        [Test]
        public void GetSunrise_OnValidDate_ReturnsExpectedTime()
        {
            Mock<ISolarService> serviceMock = GetMockService();

            var calculator = new SunsetCalculator();
            calculator.Service = serviceMock.Object;

            DateTime targetDate = new DateTime(2016, 09, 22);
            DateTime expected = new DateTime(2016, 09, 22, 6, 39, 53);

            DateTime result = calculator.GetSunrise(targetDate);

            Assert.AreEqual(expected, result);
        }

        [Test]
        public void GetSunset_WhenCalledTwiceWithSameDate_ServiceIsCalledOnce()
        {
            Mock<ISolarService> serviceMock = GetMockService();

            var calculator = new SunsetCalculator();
            calculator.Service = serviceMock.Object;

            DateTime targetDate = new DateTime(2016, 09, 22);

            calculator.GetSunset(targetDate);
            calculator.GetSunset(targetDate);

            serviceMock.Verify(
                s => s.GetServiceData(targetDate), Times.Once);
        }

        [Test]
        public void GetSunrise_WhenCalledTwiceWithSameDate_ServiceIsCalledOnce()
        {
            Mock<ISolarService> serviceMock = GetMockService();

            var calculator = new SunsetCalculator();
            calculator.Service = serviceMock.Object;

            DateTime targetDate = new DateTime(2016, 09, 22);

            calculator.GetSunrise(targetDate);
            calculator.GetSunrise(targetDate);

            serviceMock.Verify(
                s => s.GetServiceData(targetDate), Times.Once);
        }
    }
}
