﻿using Moq;
using NUnit.Framework;
using Sunset.Interface;
using Sunset.Library;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SunsetLibrary.Tests
{
    [TestFixture]
    public class SunsetCalculatorTests
    {
        string goodResult = "{\"results\":{\"sunrise\":\"6:39:53 AM\",\"sunset\":\"6:48:32 PM\",\"solar_noon\":\"12:44:12 PM\",\"day_length\":\"12:08:38.8300000\"},\"status\":\"OK\"}";
        string badResult = "{\"results\":null,\"status\":\"ERROR\"}";

        [Test]
        public void SunsetCalculator_OnCreation_MatchesInterface()
        {
            var calculator = new SunsetCalculator();
            Assert.IsInstanceOf<ISunsetCalculator>(calculator);
        }

        [Test]
        public void CheckStatus_OnValidStatus_ReturnsTrue()
        {
            bool result = SunsetCalculator.CheckStatus(goodResult);
            Assert.IsTrue(result);
        }

        [Test]
        public void CheckStatus_OnErrorStatus_ReturnsFalse()
        {
            bool result = SunsetCalculator.CheckStatus(badResult);
            Assert.IsFalse(result);
        }

        [Test]
        public void ParseSunsetString_OnValidData_ReturnsExpectedValue()
        {
            string expected = "6:48:32 PM";
            string result = SunsetCalculator.ParseSunsetString(goodResult);
            Assert.AreEqual(expected, result);
        }

        [Test]
        public void ParseSunsetString_OnErrorDate_ThrowsArgumentException()
        {
            try
            {
                SunsetCalculator.ParseSunsetString(badResult);
                Assert.Fail("ArgumentException not thrown");
            }
            catch (ArgumentException)
            {
                Assert.Pass();
            }
        }

        [Test]
        public void ToLocalTime_OnValidTimeString_ReturnsExpectedDateTime()
        {
            string timeString = "6:48:32 PM";
            DateTime targetDate = new DateTime(2016, 09, 22);
            DateTime expected = new DateTime(2016, 09, 22, 18, 48, 32);

            DateTime result = SunsetCalculator.ToLocalTime(targetDate, timeString);

            Assert.AreEqual(expected, result);
        }

        [Test]
        public void GetSunset_OnValidDate_ReturnsExpectedTime()
        {
            Mock<ISolarService> serviceMock = new Mock<ISolarService>();
            serviceMock.Setup(s => s.GetServiceData(It.IsAny<DateTime>()))
                .Returns(goodResult);

            var calculator = new SunsetCalculator();
            calculator.Service = serviceMock.Object;

            DateTime targetDate = new DateTime(2016, 09, 22);
            DateTime expected = new DateTime(2016, 09, 22, 18, 48, 32);

            DateTime result = calculator.GetSunset(targetDate);

            Assert.AreEqual(expected, result);
        }

        [Test]
        public void GetSunset_WhenCalledTwiceWithSameDate_ServiceIsCalledOnce()
        {
            Mock<ISolarService> serviceMock = new Mock<ISolarService>();
            serviceMock.Setup(s => s.GetServiceData(It.IsAny<DateTime>()))
                .Returns(goodResult);

            var calculator = new SunsetCalculator();
            calculator.Service = serviceMock.Object;

            DateTime targetDate = new DateTime(2016, 09, 22);

            calculator.GetSunset(targetDate);
            calculator.GetSunset(targetDate);

            serviceMock.Verify(
                s => s.GetServiceData(targetDate), Times.Once);
        }
    }
}
